(* ****************************************      *)
(*   Programm zur Darstellung des auf der        *)
(*                DSM 860                        *)
(*       berechneten Apfelmnnchens              *)
(*        Copyright 1991, Mnchen                *)
(*          Rolf-Dieter Klein                    *)
(*  Beispiel fuer ein Hybrid-Programm            *)
(*                                               *)
(*  Heavily modified 2012 by Axel Muhr for       *)
(*  16bit DSM card and higher res/color          *)
(*  see                                          *)
(* http://www.geekdot.com/putting-it-to-use.html *)
(*	for more info 													     *)
(* ********************************************* *)

program mandmain;

uses crt,graph, dos;
{$M 16384, 0, 100000}
{$R-}
{$I-}
{$G-}
{$X-}
{$O-}
{$V-}
{$A+}
{$X-}

{$i svgautil.inc}

const
 resxdir = 640;                 (* Anzahl der Pixel in X-Richtung *)
 resydir = 480;                 (* Anzahl der Pixel in Y-Richtung *)
 maxara = 1000;			(* Maximale Anzahl der rckgngigmachbaren Zoom-Vorgnge *)
 {--- Timer-Ticker ---}
 TimerSource     = 1193182;                 { Grundfrequenz des Timers in Hz }
 TimeSlice       = 0.010;                   { Timer-Takt. Nie > 0.055 !! }
 { Increments pro Timer-Takt }
 TickIncs        = Round (TimerSource * TimeSlice);

type merker = record cxma,cxmi,cyma,cymi : double ; end;
     fldstru = array[0..479] of byte; (* war 511*)

var y,b,a,ah2,graphdriver,graphmode,GrErr:integer;
    xi:integer absolute $dc00:$252;
    itermax:integer absolute $dc00:$254;
    flag:byte absolute $dc00:$250;
    feld:fldstru absolute $dc00:0;
    feld1:fldstru;
    cxmax: double absolute $dc00:0000;
    cxmin: double absolute $dc00:0008;
    cymax: double absolute $dc00:0016;
    cymin: double absolute $dc00:0024;
    ch : char;
    cxmax1: double;
    cxmin1: double;
    cymax1: double;
    cymin1: double;
    cxmax2: double;
    cxmin2: double;
    cymax2: double;
    cymin2: double;
    xmax: integer;
    xmin: integer;
    ymax: integer;
    ymin: integer;
    merkall: array[1..maxara] of merker;
    merkcntmax: integer;
    merkcnt: integer;
    aa:string;
    (* Timer stuff *)
    T		: longint;
    ExitSave    : POINTER;    { Merker fr Exit-Prozedur }
    SaveInt08   : POINTER;    { Alter Interruptvektor 08h }
    SystemTicks : LONGINT;    { Zhlt die TimeSlice-Ticks }
    TotalTime   : LONGINT;    { Zhler fr Gesamzeit des Tests }

{ --- Inlines --- }

PROCEDURE SetTimer0 (TimerWert : WORD);       { Timer 0 einstellen }
  INLINE ($B0/$34/           { mov     al, 34h }
          $E6/$43/           { out     43h, al }
          $58/               { pop     ax      }
          $E6/$40/           { out     40h, al }
          $86/$C4/           { xchg    al, ah  }
          $E6/$40);          { out     40h, al }

PROCEDURE ResetTimer0;       { Timer 0 rcksetzen }
  INLINE ($B0/$34/           { mov     al, 34h }
          $E6/$43/           { out     43h, al }
          $B0/$00/           { mov     al, 00h }
          $E6/$40/           { out     40h, al }
          $E6/$40);          { out     40h, al }


{ -------------------------------------------------------------------------- }
{ Hilfsprozeduren zur Zeitmessung }


PROCEDURE BGIBenchExit; FAR;
{ Stellt bei Programmende den orginalen Int8-Vektor sowie die ursprngliche
  Timer-Frequenz wieder her. Wird auch nach Ablauf der Messung aufgerufen.
}
BEGIN
  ExitProc := ExitSave;
  ResetTimer0;
  SetIntVec ($08, SaveInt08);
END;




PROCEDURE Int8Handler; FAR; ASSEMBLER;
{ Bildet den orginalen Timer in Software nach, zhlt intern aber Ticks der
  Periode TimeSlice.
}
CONST
  Ticks : WORD = 0;

ASM
  { Bentigte Register retten/setzen }
  push    ax
  push    ds
  mov     ax, Seg @Data
  mov     ds, ax

  { Den internen Zeitzhler hochzhlen }
  add     [WORD Ptr SystemTicks], 1
  adc     [WORD Ptr SystemTicks+2], 0 { berlauf }

  { Den Software-Timer hochzhlen }
  mov     ax, TickIncs                { Ticks pro TimeSlice }
  add     [Ticks], ax                 { auf den Software-Timer addieren }
  jnc     @L1                         { Springe wenn kein berlauf }

  { berlauf des Software-Timers. Den alten Int08 aufrufen, dann Ende }
  pushf
  call    [SaveInt08]                 { Int 8 simulieren }
  jmp     @L2

  { Kein berlauf, dem Interrupt-Controller ein EOI schicken, dann Ende }
@L1:
  mov     al, 20h
  out     20h, al

  { Ende }
@L2:
  pop     ds
  pop     ax
  iret
END;



PROCEDURE InstallInt8Handler;
{ Klinkt eine eigene Routine in den System-Ticker ein und beschleunigt
  den Ticker auf den Wert TimeSlice.
}
BEGIN
  { Interrupt-Vektor 8 holen }
  GetIntVec ($08, SaveInt08);
  { Exit-Prozedur einklinken }
  ExitSave := ExitProc;
  ExitProc := @BGIBenchExit;
  { Int 8 einklinken }
  SetIntVec ($08, @Int8Handler);
  { Ticker beschleunigen }
  SetTimer0 (TickIncs);
END;




FUNCTION SystemTime : LONGINT; ASSEMBLER;
{ Holt den Ticker-Zhler. Ein Direkt-Zugriff auf die Variable ist nicht
  mglich, da diese nicht in einem Rutsch gelesen wird (keine atomare
  Operation). Deshalb Umweg ber diese Routine, die whrend des Zugriffs
  die Interrupts sperrt.
}
ASM
  pushf                                    { I-Flag retten }
  cli                                      { interrupts off }
  mov     ax, [WORD Ptr SystemTicks]       { Low Word holen }
  mov     dx, [WORD Ptr SystemTicks+2]     { High Word holen }
  popf                                     { Altes I-Flag restaurieren }
END;


(* simple One-Processor-Procedures begin *)

procedure oneondsm;

var i:integer;
 checkvar : longint absolute $dc00:0;
 runflag : byte absolute $dc00:$210;   (* resetstate, mon, run *)
begin
 checkvar := $1234a15a;
 if checkvar = $1234a15a then exit;  (* ok some enabled already *)
 i := 0;
 repeat
  port[$300+i] := $c2; (* Set IO-MEM Window to 0xDC00-DCFF *)
  checkvar := $1234a25a;
  if checkvar <> $1234a25a then port[300+i] := 0;
  i := i + 1;
 until (checkvar = $1234a25a) or (i = 32);
 if i = 32 then begin
  closegraph;
  writeln('ERROR: NO DSM-860 found.');
  halt(1);
 end;
 if runflag = $55 then begin
  closegraph;
  writeln('DSM 860 in reset state, please call load860 mon.p68 monb16.p68');
  halt(1);
 end else if runflag = $A1 then begin
  closegraph;
  writeln('DSM 860, monitor running, but no user program.');
  writeln('         use server -1 <program> to start the demo program.');
  halt(1);
 end else if runflag <> $a0 then begin
  closegraph;
  writeln('DSM 860, illegale state, use load860 to reset CPU');
  halt(1);
 end;
end;

procedure alloffdsm;
var i:integer;
begin
 for i:=0 to 31 do port[$300+i] := 0;   (* total off *)
end;

(* simple One-Processor-Procedures end *)

procedure infoiter;
(* Ausgabe der Anzahl der Aktuellen Iterations Anzahl *)
 var txt:string;

begin
 setbkcolor(black);             (* Ausgabe eines Schwarzen, *)
 setcolor(white);
 setfillstyle(1,black);         (* gefllten *)
 bar(0,0,16*8,8);           (* Rechtecks am linken, oberen Bildschirmrand *)
 outtextxy(0,0,'Iterationen:'); (* Text 'Iterationen:' ausgeben *)
 str(itermax,txt);              (* Iterations Anzahl in String umwandeln *)
 outtextxy(12*8,0,txt);         (* und ausgeben *)
end;

procedure info;                 (* Alle Bildinformationen ausgeben *)
 var txt:string;

begin
 setbkcolor(black);             (* Ausgabe eines Schwarzen, *)
 setcolor(white);
 setfillstyle(1,black);         (* gefllten *)
 bar(0,0,25*8,58);
   (* aber etwas greren Rechtecks am linken oberen Bildschirmrand *)
 infoiter;                      (* Anzahl der Iterationen ausgeben *)
 outtextxy(0,10,'cxmax:');      (* Text 'cxmax:' ausgeben *)
 str(cxmax1:1:16,txt);          (* cxmax in einen String umwandeln *)
 outtextxy(6*8,10,txt);         (* und ausgeben *)
 outtextxy(0,20,'cxmin:');      (* Text 'cxmin:' ausgeben *)
 str(cxmin1:1:16,txt);          (* cxmin in einen String umwandeln *)
 outtextxy(6*8,20,txt);         (* und ausgeben *)
 outtextxy(0,30,'cymax:');      (* Text 'cymax:' ausgeben *)
 str(cymax1:1:16,txt);          (* cymax in einen String umwandeln *)
 outtextxy(6*8,30,txt);         (* und ausgeben *)
 outtextxy(0,40,'cymin:');      (* Text 'cymin:' ausgeben *)
 str(cymin1:1:16,txt);          (* cymin in einen String umwandeln *)
 outtextxy(6*8,40,txt);         (* und ausgeben *)
 outtextxy(0,50,'Merker:');     (* Text 'Merker:' ausgeben *)
 str(merkcnt,txt);              (* Merkerzhler in einen String umwandeln *)
 outtextxy(7*8,50,txt);         (* und ausgeben *)
end;

procedure rahmen;
(* Rahmen um den selektierten Ausschnitt zeichnen *)
begin
 setwritemode(1);               (* XOR-Schreibmodus *)
 setcolor(white);               (* Schreibfarbe wei *)
 rectangle(xmin,ymin,xmax,ymax);(* Rechteck des Rahmens ziehen *)
 setwritemode(0);               (* zurck in normalen schreibmode *)
end;

procedure scancode(ch:char);
begin
 rahmen;                        (* alten Rahmen lschen *)
 case ch of
  chr(75):if xmax > 0 then xmax:=xmax-5;   (* Pfeiltaste Links *)
  chr(77):if xmax < (resxdir-1) then xmax:=xmax+5; (* Pfeiltaste Rechts *)
  chr(72):if ymax > 0 then ymax:=ymax-5;   (* Pfeiltaste Rauf *)
  chr(80):if ymax < (resydir-1) then ymax:=ymax+5; (* Pfeiltaste Runter *)
  chr(59):if xmin > 0 then xmin:=xmin-5;   (* Funktionstaste F1 *)
  chr(60):if xmin < (resxdir-1) then xmin:=xmin+5; (* Funktionstaste F2 *)
  chr(61):if ymin > 0 then ymin:=ymin-5;   (* Funktionstaste F3 *)
  chr(62):if ymin < (resydir-1) then ymin:=ymin+5; (* Funktionstaste F4 *)
 end;
 rahmen;                        (* neuen Rahmen zeichnen *)
 (* neue Ausschnittswerte berechnen *)
 cxmax1:=cxmin2+((cxmax2-cxmin2)*xmax)/(resxdir-1);
 cymax1:=cymin2+((cymax2-cymin2)*ymax)/(resydir-1);
 cxmin1:=cxmin2+((cxmax2-cxmin2)*xmin)/(resxdir-1);
 cymin1:=cymin2+((cymax2-cymin2)*ymin)/(resydir-1);
end;

begin
writeln('allOffDSM');   
 alloffdsm;
writeln('oneOnDSM'); 
 oneondsm;
 (*graphdriver := detect;          Graphikkarte suchen *)
 graphdriver := InstallUserDriver ('SVGA256', 2);
 
(* 0) 320x200x256
1) 640x400x256
2) 640x480x256
3) 800x600x256
4) 1024x768x256
5) 640x350x256
6) 1280x1024x256

*)
 
  GraphDriver := Detect;
  GraphMode := 2;
  InitGraph(GraphDriver,GraphMode,'');
  GrErr := GraphResult;             { preserve error return }
  if GrErr AND $80 = $80 then
     GrErr := GrErr OR $ff00;
  if GrErr <> grOK then             { error? }
  begin
    Writeln('Graphics error: ', GraphErrorMsg(GrErr));
    Halt(1);
  end; 

 cleardevice;                   (* Bildschirm lschen *) 
 itermax:=100;                  (* Iterationen voreinstellen *)
 delay(100);                    (* 100 ms warten *)
 (* Koordinaten des Apfelmnnchens voreinstellem *)
 cxmax1 := 0.6;
 cxmin1 := -2.0;
 cymax1 := 1.2;
 cymin1 := -1.2;
 merkcntmax := 1;               (* Start bei Merker = 1 *)
 merkcnt := 1;
 T := SystemTime;		(* Start Timer *)
 repeat
  cleardevice;                  (* Bildschirm lschen *)
  repeat until (flag = $aa) or (flag = 0); (* Warten bis dsm860 bereit *)
  xi := 0;                      (* Spaltenzhler auf 0 *)
  cxmax := cxmax1;              (* Werte an i860 bertragen *)
  cxmin := cxmin1;
  cymax := cymax1;
  cymin := cymin1;
  merkall[merkcntmax].cxma := cxmax1; (* alte Werte merken *)
  merkall[merkcntmax].cxmi := cxmin1;
  merkall[merkcntmax].cyma := cymax1;
  merkall[merkcntmax].cymi := cymin1;
  if merkcntmax < maxara then merkcntmax := merkcntmax + 1; (* Zhler +1 *)
  if merkcnt = merkcntmax-1 then merkcnt := merkcntmax;
  flag := 2;                    (* Parameter bergeben *)
  
  repeat until (flag = 0);      (* warten bis fertig *)
  (* writeln('Ok, sende 1 fuer start'); *)
  flag := 1;                    (* i860 starten *)
  a:=1;
  repeat
   repeat until (flag = 0); (* warten bis fertig *)
   feld1:=feld;
   xi := a;
   flag:=1;                     (* neu starten *)
   for b:=0 to 479 do begin
	putpixel(a,b,feld1[b]); 
   end;
   a:=a+1;                      (* Spaltenzhler um 1 erhhen *)
   
(*   setcolor(white);
   setfillstyle(1,black);         
   bar(0,0,10*8,20);
   str(graphdriver, aa);
   outtextxy(5,10,aa);
   str(flag,aa);
   outtextxy(50,10,aa); *)

  until (a=639) or keypressed;  (* solange, bis fertig oder Abbruch *)

  T := SystemTime - T;		(* Read Timer*)
  setcolor(white);
  setfillstyle(1,black);         (* gefllten *)
  bar(0,0,10*8,20);
  str(((TimeSlice * T)) * 1000 :6:2, aa);
  outtextxy(5,10,aa);
  
  xmin:=0;                      (* Rahmenkoordinaten festlegen *)
  ymin:=0;
  xmax:=resxdir;
  ymax:=resydir;
  rahmen;                       (* Rahmen zeichnen *)
  cxmax2 := cxmax1;             (* alte Werte speichern *)
  cxmin2 := cxmin1;
  cymax2 := cymax1;
  cymin2 := cymin1;
  repeat
   ch := readkey;               (* Zeichen von Tastatur einlesen *)
   case ch of
    chr(0):scancode(readkey);
    '+':begin                   (* wenn '+', dann Iterationen um 10 erhhen *)
	 itermax:=abs(itermax+10) mod 9999;
	 infoiter;              (* Anzahl der Iterationen anzeigen *)
	end;
    '*':begin                   (* wenn '*', dann Iter. um 100 erhhen *)
	 itermax:=abs(itermax+100) mod 9999;
	 infoiter;              (* Anzahl der Iterationen anzeigen *)
	end;
    '-':begin                   (* wenn '-', dann Iter. um 10 verringern *)
	 itermax:=abs(itermax-10) mod 9999;
	 infoiter;              (* Anzahl der Iterationen anzeigen *)
	end;
    '/':begin                   (* wenn '/', dann Iter. um 100 verringern *)
	 itermax:=abs(itermax-100) mod 9999;
	 infoiter;              (* Anzahl der Iterationen anzeigen *)
	end;
    'b': begin                  (* Zurckschreiten *)
	 if merkcnt > 1 then begin
	  merkcnt := merkcnt - 1; (* Merkerzhler verringern *)
	  cxmax1 := merkall[merkcnt].cxma; (* und Werte bernehmen *)
	  cxmin1 := merkall[merkcnt].cxmi;
	  cymax1 := merkall[merkcnt].cyma;
	  cymin1 := merkall[merkcnt].cymi;
	 end;
	 info;
	end;
    'f': begin                  (* Vorwrtsschreiten *)
	 if merkcnt < merkcntmax-1 then begin
	  merkcnt := merkcnt + 1; (* Merkerzhler verringern *)
	  cxmax1 := merkall[merkcnt].cxma; (* und Werte bernehmen *)
	  cxmin1 := merkall[merkcnt].cxmi;
	  cymax1 := merkall[merkcnt].cyma;
	  cymin1 := merkall[merkcnt].cymi;
	 end;
	 info;                  (* neue Koordinaten ausgeben *)
	end;
    'i':info;                   (* Bildschirminformationen ausgeben *)
   end;
  until ch in [chr($1b),chr($d)]; (* bis <ESC> oder <RETURN> *)
 until ch=chr($1b);             (* bei <ESC> Programm verlassen *)
 closegraph;                    (* Graphokmode verlassen *)
 alloffdsm;
end.
